// simple.frag

uniform sampler2D textureImage;
uniform vec4 levelMin;
uniform vec4 levelMax;
uniform vec4 exposure;
uniform vec4 gamma;
uniform int hueOffset;
uniform int saturation;

void main (void)
{
	gl_FragColor = texture2D(textureImage, gl_TexCoord[0].st);
	gl_FragColor = (gl_FragColor-levelMin) / (levelMax - levelMin);

	if (hueOffset != 0 || saturation != 0){
	
	    float R = gl_FragColor.r;
	    float G = gl_FragColor.g;
	    float B = gl_FragColor.b;
	    
	    float max = R;
	    int maxType = 0;
	    if (G > max){ max = G; maxType = 1; }
	    if (B > max){ max = B; maxType = 2; }
	    
	    float min = R;
	    if (G < min){ min = G; }
	    if (B < min){ min = B; }
	    
	    float H = 0, S = 0, V = 0;
	    if      (max == 0)  { H = 0; S = 0; V = 0; }
	    else if (max == min){ H = 0; S = 0; V = max;}
	    else {
		V = max;
		S = ((float)(max - min)) / max;
		switch(maxType){
		case 0:
		    H = ((float)(G - B))/(max - min) + 0; break;
		case 1:
		    H = ((float)(B - R))/(max - min) + 2; break;
		case 2:
		    H = ((float)(R - G))/(max - min) + 4; break;
		}
	    }
	    
	    S *= (float)saturation/100 + 1;
	    if (S > 1) S = 1;

	    H += (float)hueOffset/60;
	    
	    while (H < 0) H += 6;
	    H = fmod(H, 6);
	    int Hi = ((int)H) % 6;
	    float f = (float)H - Hi;
	    
	    switch (Hi){
	    case 0:
		R = V;
		G = V * ( 1.0 - ( 1.0 - f) * S);
		B = V * ( 1.0 - S);
		break;
	    case 1:
		R = V * ( 1.0 - f * S);
		G = V;
		B = V * ( 1.0 - S);
		break;
	    case 2:
		R = V * ( 1.0 - S);
		G = V;
		B = V * ( 1.0 - ( 1.0 - f) * S);
		break;
	    case 3:
		R = V * ( 1.0 - S);
		G = V * ( 1.0 - f * S);
		B = V;
		break;
	    case 4:
		R = V * ( 1.0 - ( 1.0 - f) * S);
		G = V * ( 1.0 - S);
		B = V;
		break;
	    case 5:
		R = V;
		G = V * ( 1.0 - S);
		B = V * ( 1.0 - f * S);
		break;
	    }
	    
	    gl_FragColor = vec4(R, G, B, 1.0);
	}

	if (gamma[0] == 0) return;

	gl_FragColor = pow(gl_FragColor, gamma);
}
